@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2016, Wolf Vollprecht, Johan Mabille and Sylvain Corlay    *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -31,6 +33,7 @@ namespace xsimd
         using self_type = batch<double, 2>;
         using base_type = simd_batch<self_type>;
         using storage_type = typename base_type::storage_type;
+        using batch_bool_type = typename base_type::batch_bool_type;
 
         batch();
         explicit batch(double d);
@@ -43,6 +46,9 @@ namespace xsimd
         batch(const storage_type& rhs);
         batch& operator=(const storage_type& rhs);
 
+        batch(const batch_bool_type& rhs);
+        batch& operator=(const batch_bool_type& rhs);
+        
         operator storage_type() const;
 
         XSIMD_DECLARE_LOAD_STORE_ALL(double, 2)
@@ -98,6 +104,19 @@ namespace xsimd
         return *this;
     }
 
+    inline batch<double, 2>::batch(const batch_bool_type& rhs)
+        : base_type(vreinterpretq_f64_u64(vandq_u64(rhs,
+                                                    vreinterpretq_u64_f64(batch(1.)))))
+    {
+    }
+
+    inline batch<double, 2>& batch<double, 2>::operator=(const batch_bool_type& rhs)
+    {
+        this->m_value = vreinterpretq_f64_u64(vandq_u64(rhs,
+                                                    vreinterpretq_u64_f64(batch(1.))));
+        return *this;
+    }
+
     inline batch<double, 2>& batch<double, 2>::load_aligned(const int8_t* src)
     {
         int8x8_t tmp = vld1_s8((const int8_t*)src);
@@ -127,6 +146,8 @@ namespace xsimd
         return *this;
     }
 
+    XSIMD_DEFINE_LOAD_STORE(double, 2, bool, XSIMD_DEFAULT_ALIGNMENT)
+
     inline batch<double, 2>& batch<double, 2>::load_unaligned(const uint8_t* src)
     {
         return load_aligned(src);
