@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2016, Wolf Vollprecht, Johan Mabille and Sylvain Corlay    *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -36,6 +38,7 @@ namespace xsimd
         using self_type = batch<float, 4>;
         using base_type = simd_batch<self_type>;
         using storage_type = typename base_type::storage_type;
+        using batch_bool_type = typename base_type::batch_bool_type;
 
         batch();
         explicit batch(float d);
@@ -48,6 +51,9 @@ namespace xsimd
         batch(const storage_type& rhs);
         batch& operator=(const storage_type& rhs);
 
+        batch(const batch_bool_type& rhs);
+        batch& operator=(const batch_bool_type& rhs);
+
         operator storage_type() const;
 
         XSIMD_DECLARE_LOAD_STORE_ALL(float, 4)
@@ -103,6 +109,18 @@ namespace xsimd
         return *this;
     }
 
+    inline batch<float, 4>::batch(const batch_bool_type& rhs)
+        : base_type(vreinterpretq_f32_u32(vandq_u32(rhs, 
+                                                    vreinterpretq_u32_f32(batch(1.f)))))
+    {
+    }
+
+    inline batch<float, 4>& batch<float, 4>::operator=(const batch_bool_type& rhs)
+    {
+        this->m_value = vreinterpretq_f32_u32(vandq_u32(rhs, vreinterpretq_u32_f32(batch(1.f))));
+        return *this;
+    }
+
     inline batch<float, 4>& batch<float, 4>::load_aligned(const int8_t* src)
     {
         int8x8_t tmp = vld1_s8((const int8_t*)src);
@@ -118,6 +136,8 @@ namespace xsimd
         return load_aligned(src);
     }
 
+    XSIMD_DEFINE_LOAD_STORE(float, 4, bool, XSIMD_DEFAULT_ALIGNMENT)
+
     inline batch<float, 4>& batch<float, 4>::load_aligned(const uint8_t* src)
     {
         uint8x8_t tmp = vld1_u8((const uint8_t*)src);
@@ -272,7 +292,7 @@ namespace xsimd
 
     inline void batch<float, 4>::store_unaligned(int8_t* dst) const
     {
-        store_unaligned(dst);
+        store_aligned(dst);
     }
 
     inline void batch<float, 4>::store_aligned(uint8_t* dst) const
@@ -298,7 +318,7 @@ namespace xsimd
 
     inline void batch<float, 4>::store_unaligned(int16_t* dst) const
     {
-        store_unaligned(dst);
+        store_aligned(dst);
     }
 
     inline void batch<float, 4>::store_aligned(uint16_t* dst) const
