@@ -1,6 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay,  Wolf Vollprecht and  *
-* Martin Renou                                                             *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -10,8 +10,11 @@
 #ifndef XTENSOR_MANIPULATION_HPP
 #define XTENSOR_MANIPULATION_HPP
 
+#include "xbuilder.hpp"
 #include "xstrided_view.hpp"
 #include "xutils.hpp"
+#include "xtensor_config.hpp"
+#include "xrepeat.hpp"
 
 namespace xt
 {
@@ -37,15 +40,63 @@ namespace xt
     template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL, class E>
     auto flatten(E&& e);
 
+    template <layout_type L, class T>
+    auto flatnonzero(const T& arr);
+
     template <class E>
     auto trim_zeros(E&& e, const std::string& direction = "fb");
 
     template <class E>
     auto squeeze(E&& e);
 
-    template <class E, class S, class Tag = check_policy::none, std::enable_if_t<!std::is_integral<S>::value, int> = 0>
+    template <class E, class S, class Tag = check_policy::none, std::enable_if_t<!xtl::is_integral<S>::value, int> = 0>
     auto squeeze(E&& e, S&& axis, Tag check_policy = Tag());
 
+    template <class E>
+    auto expand_dims(E&& e, std::size_t axis);
+
+    template <std::size_t N, class E>
+    auto atleast_Nd(E&& e);
+
+    template <class E>
+    auto atleast_1d(E&& e);
+
+    template <class E>
+    auto atleast_2d(E&& e);
+
+    template <class E>
+    auto atleast_3d(E&& e);
+
+    template <class E>
+    auto split(E& e, std::size_t n, std::size_t axis = 0);
+
+    template <class E>
+    auto hsplit(E& e, std::size_t n);
+
+    template <class E>
+    auto vsplit(E& e, std::size_t n);
+
+    template <class E>
+    auto flip(E&& e, std::size_t axis);
+
+    template <std::ptrdiff_t N = 1, class E>
+    auto rot90(E&& e, const std::array<std::ptrdiff_t, 2>& axes = {0, 1});
+
+    template<class E>
+    auto roll(E&& e, std::ptrdiff_t shift);
+
+    template<class E>
+    auto roll(E&& e, std::ptrdiff_t shift, std::ptrdiff_t axis);
+
+    template<class E>
+    auto repeat(E&& e, std::size_t repeats, std::size_t axis);
+
+    template<class E>
+    auto repeat(E&& e, const std::vector<std::size_t>& repeats, std::size_t axis);
+
+    template<class E>
+    auto repeat(E&& e, std::vector<std::size_t>&& repeats, std::size_t axis);
+
     /****************************
      * transpose implementation *
      ****************************/
@@ -70,7 +121,7 @@ namespace xt
         {
             if (l != layout_type::row_major && l != layout_type::column_major)
             {
-                throw transpose_error("cannot compute transposed layout of dynamic layout");
+                XTENSOR_THROW(transpose_error, "cannot compute transposed layout of dynamic layout");
             }
             return transpose_layout_noexcept(l);
         }
@@ -80,7 +131,7 @@ namespace xt
         {
             if (sequence_size(permutation) != e.dimension())
             {
-                throw transpose_error("Permutation does not have the same size as shape");
+                XTENSOR_THROW(transpose_error, "Permutation does not have the same size as shape");
             }
 
             // permute stride and shape
@@ -97,7 +148,7 @@ namespace xt
             {
                 if (std::size_t(permutation[i]) >= e.dimension())
                 {
-                    throw transpose_error("Permutation contains wrong axis");
+                    XTENSOR_THROW(transpose_error, "Permutation contains wrong axis");
                 }
                 size_type perm = static_cast<size_type>(permutation[i]);
                 temp_shape[i] = e.shape()[perm];
@@ -115,8 +166,7 @@ namespace xt
                 new_layout = transpose_layout_noexcept(e.layout());
             }
 
-            using view_type = typename select_strided_view<std::decay_t<E>>::template type<xclosure_t<E>, shape_type>;
-            return view_type(std::forward<E>(e), std::move(temp_shape), std::move(temp_strides), get_offset(e), new_layout);
+            return strided_view(std::forward<E>(e), std::move(temp_shape), std::move(temp_strides), get_offset<XTENSOR_DEFAULT_LAYOUT>(e), new_layout);
         }
 
         template <class E, class S>
@@ -129,7 +179,7 @@ namespace xt
                 {
                     if (permutation[i] == permutation[j])
                     {
-                        throw transpose_error("Permutation contains axis more than once");
+                        XTENSOR_THROW(transpose_error, "Permutation contains axis more than once");
                     }
                 }
             }
@@ -146,9 +196,9 @@ namespace xt
         inline void compute_transposed_strides(E&&, const S& shape, X& strides)
         {
             // In the case where E does not have a data interface, the transposition
-            // makes use of a flat storage adaptor that has layout XTENSOR_DEFAULT_LAYOUT
+            // makes use of a flat storage adaptor that has layout XTENSOR_DEFAULT_TRAVERSAL
             // which should be the one inverted.
-            layout_type l = transpose_layout(XTENSOR_DEFAULT_LAYOUT);
+            layout_type l = transpose_layout(XTENSOR_DEFAULT_TRAVERSAL);
             compute_strides(shape, l, strides);
         }
     }
@@ -171,8 +221,7 @@ namespace xt
 
         layout_type new_layout = detail::transpose_layout_noexcept(e.layout());
 
-        using view_type = typename select_strided_view<std::decay_t<E>>::template type<xclosure_t<E>, shape_type>;
-        return view_type(std::forward<E>(e), std::move(shape), std::move(strides), detail::get_offset(e), new_layout);
+        return strided_view(std::forward<E>(e), std::move(shape), std::move(strides), detail::get_offset<XTENSOR_DEFAULT_TRAVERSAL>(e), new_layout);
     }
 
     /**
@@ -189,28 +238,41 @@ namespace xt
     }
 
     /// @cond DOXYGEN_INCLUDE_SFINAE
-#ifdef X_OLD_CLANG
-    template <class E, class I, class Tag = check_policy::none>
-    inline auto transpose(E&& e, std::initializer_list<I> permutation, Tag check_policy = Tag())
-    {
-        dynamic_shape<I> perm(permutation);
-        return detail::transpose_impl(std::forward<E>(e), std::move(perm), check_policy);
-    }
-#else
     template <class E, class I, std::size_t N, class Tag = check_policy::none>
     inline auto transpose(E&& e, const I(&permutation)[N], Tag check_policy = Tag())
     {
         return detail::transpose_impl(std::forward<E>(e), permutation, check_policy);
     }
-#endif
     /// @endcond
 
-    /***************************
-     * ravel and flatten views *
-     ***************************/
+    /************************************
+     * ravel and flatten implementation *
+     ************************************/
 
-    template <class I, class CI>
-    class xiterator_adaptor;
+    namespace detail
+    {
+        template <class E, layout_type L>
+        struct expression_iterator_getter
+        {
+            using iterator = decltype(std::declval<E>().template begin<L>());
+            using const_iterator = decltype(std::declval<E>().template cbegin<L>());
+
+            inline static iterator begin(E& e)
+            {
+                return e.template begin<L>();
+            }
+
+            inline static const_iterator cbegin(E& e)
+            {
+                return e.template cbegin<L>();
+            }
+
+            inline static auto size(E& e)
+            {
+                return e.size();
+            }
+        };
+    }
 
     /**
      * Returns a flatten view of the given expression. No copy is made.
@@ -223,11 +285,12 @@ namespace xt
     inline auto ravel(E&& e)
     {
         using iterator = decltype(e.template begin<L>());
-        using const_iterator = decltype(e.template cbegin<L>());
-        using adaptor_type = xiterator_adaptor<iterator, const_iterator>;
+        using iterator_getter = detail::expression_iterator_getter<std::remove_reference_t<E>, L>;
+        auto size = e.size();
+        auto adaptor = make_xiterator_adaptor(std::forward<E>(e), iterator_getter());
         constexpr layout_type layout = std::is_pointer<iterator>::value ? L : layout_type::dynamic;
-        using type = xtensor_adaptor<adaptor_type, 1, layout, extension::get_expression_tag_t<E>>;
-        return type(adaptor_type(e.template begin<L>(), e.template cbegin<L>(), e.size()), { e.size() });
+        using type = xtensor_view<decltype(adaptor), 1, layout, extension::get_expression_tag_t<E>>;
+        return type(std::move(adaptor), { size });
     }
 
     /**
@@ -246,6 +309,23 @@ namespace xt
         return ravel<L>(std::forward<E>(e));
     }
 
+    /**
+     * @brief return indices that are non-zero in the flattened version of arr,
+     * equivalent to nonzero(ravel<layout_type>(arr))[0];
+     *
+     * @param arr input array
+     * @return indices that are non-zero in the flattened version of arr
+     */
+    template <layout_type L, class T>
+    inline auto flatnonzero(const T& arr)
+    {
+        return nonzero(ravel<L>(arr))[0];
+    }
+
+    /*****************************
+     * trim_zeros implementation *
+     *****************************/
+
     /**
      * Trim zeros at beginning, end or both of 1D sequence.
      *
@@ -278,6 +358,10 @@ namespace xt
         return strided_view(std::forward<E>(e), { range(begin, end) });
     }
 
+    /**************************
+     * squeeze implementation *
+     **************************/
+
     /**
      * Returns a squeeze view of the given expression. No copy is made.
      * Squeezing an expression removes dimensions of extent 1.
@@ -292,12 +376,11 @@ namespace xt
         dynamic_shape<std::ptrdiff_t> new_strides;
         std::copy_if(e.shape().cbegin(), e.shape().cend(), std::back_inserter(new_shape),
                      [](std::size_t i) { return i != 1; });
-        decltype(auto) old_strides = detail::get_strides(e);
+        decltype(auto) old_strides = detail::get_strides<XTENSOR_DEFAULT_LAYOUT>(e);
         std::copy_if(old_strides.cbegin(), old_strides.cend(), std::back_inserter(new_strides),
                      [](std::ptrdiff_t i) { return i != 0; });
 
-        using view_type = xstrided_view<xclosure_t<E>, dynamic_shape<std::size_t>>;
-        return view_type(std::forward<E>(e), std::move(new_shape), std::move(new_strides), 0, e.layout());
+        return strided_view(std::forward<E>(e), std::move(new_shape), std::move(new_strides), 0, e.layout());
     }
 
     namespace detail
@@ -309,7 +392,7 @@ namespace xt
             dynamic_shape<std::size_t> new_shape(new_dim);
             dynamic_shape<std::ptrdiff_t> new_strides(new_dim);
 
-            decltype(auto) old_strides = detail::get_strides(e);
+            decltype(auto) old_strides = detail::get_strides<XTENSOR_DEFAULT_LAYOUT>(e);
 
             for (std::size_t i = 0, ix = 0; i < e.dimension(); ++i)
             {
@@ -320,8 +403,7 @@ namespace xt
                 }
             }
 
-            using view_type = xstrided_view<xclosure_t<E>, dynamic_shape<std::size_t>>;
-            return view_type(std::forward<E>(e), std::move(new_shape), std::move(new_strides), 0, e.layout());
+            return strided_view(std::forward<E>(e), std::move(new_shape), std::move(new_strides), 0, e.layout());
         }
 
         template <class E, class S>
@@ -331,11 +413,11 @@ namespace xt
             {
                 if (static_cast<std::size_t>(ix) > e.dimension())
                 {
-                    throw std::runtime_error("Axis argument to squeeze > dimension of expression");
+                    XTENSOR_THROW(std::runtime_error, "Axis argument to squeeze > dimension of expression");
                 }
                 if (e.shape()[static_cast<std::size_t>(ix)] != 1)
                 {
-                    throw std::runtime_error("Trying to squeeze axis != 1");
+                    XTENSOR_THROW(std::runtime_error, "Trying to squeeze axis != 1");
                 }
             }
             return squeeze_impl(std::forward<E>(e), std::forward<S>(axis), check_policy::none());
@@ -351,28 +433,19 @@ namespace xt
      * @param check_policy select check_policy. With check_policy::full(), selecting an axis
      *        which is greater than one will throw a runtime_error.
      */
-    template <class E, class S, class Tag, std::enable_if_t<!std::is_integral<S>::value, int>>
+    template <class E, class S, class Tag, std::enable_if_t<!xtl::is_integral<S>::value, int>>
     inline auto squeeze(E&& e, S&& axis, Tag check_policy)
     {
         return detail::squeeze_impl(std::forward<E>(e), std::forward<S>(axis), check_policy);
     }
 
     /// @cond DOXYGEN_INCLUDE_SFINAE
-#ifdef X_OLD_CLANG
-    template <class E, class I, class Tag = check_policy::none>
-    inline auto squeeze(E&& e, std::initializer_list<I> axis, Tag check_policy = Tag())
-    {
-        dynamic_shape<I> ax(axis);
-        return detail::squeeze_impl(std::forward<E>(e), std::move(ax), check_policy);
-    }
-#else
     template <class E, class I, std::size_t N, class Tag = check_policy::none>
     inline auto squeeze(E&& e, const I(&axis)[N], Tag check_policy = Tag())
     {
         using arr_t = std::array<I, N>;
         return detail::squeeze_impl(std::forward<E>(e), xtl::forward_sequence<arr_t, decltype(axis)>(axis), check_policy);
     }
-#endif
 
     template <class E, class Tag = check_policy::none>
     inline auto squeeze(E&& e, std::size_t axis, Tag check_policy = Tag())
@@ -381,6 +454,10 @@ namespace xt
     }
     /// @endcond
 
+    /******************************
+     * expand_dims implementation *
+     ******************************/
+
     /**
      * @brief Expand the shape of an xexpression.
      *
@@ -392,13 +469,17 @@ namespace xt
      * @return returns a ``strided_view`` with expanded dimension
      */
     template <class E>
-    auto expand_dims(E&& e, std::size_t axis)
+    inline auto expand_dims(E&& e, std::size_t axis)
     {
         xstrided_slice_vector sv(e.dimension() + 1, all());
         sv[axis] = newaxis();
         return strided_view(std::forward<E>(e), std::move(sv));
     }
 
+    /*****************************
+     * atleast_Nd implementation *
+     *****************************/
+
     /**
      * Expand dimensions of xexpression to at least `N`
      *
@@ -412,7 +493,7 @@ namespace xt
      * @return ``strided_view`` with expanded dimensions
      */
     template <std::size_t N, class E>
-    auto atleast_Nd(E&& e)
+    inline auto atleast_Nd(E&& e)
     {
         xstrided_slice_vector sv((std::max)(e.dimension(), N), all());
         if (e.dimension() < N)
@@ -437,7 +518,7 @@ namespace xt
      * @sa atleast_Nd
      */
     template <class E>
-    auto atleast_1d(E&& e)
+    inline auto atleast_1d(E&& e)
     {
         return atleast_Nd<1>(std::forward<E>(e));
     }
@@ -447,7 +528,7 @@ namespace xt
      * @sa atleast_Nd
      */
     template <class E>
-    auto atleast_2d(E&& e)
+    inline auto atleast_2d(E&& e)
     {
         return atleast_Nd<2>(std::forward<E>(e));
     }
@@ -457,11 +538,15 @@ namespace xt
      * @sa atleast_Nd
      */
     template <class E>
-    auto atleast_3d(E&& e)
+    inline auto atleast_3d(E&& e)
     {
         return atleast_Nd<3>(std::forward<E>(e));
     }
 
+    /************************
+     * split implementation *
+     ************************/
+
     /**
      * @brief Split xexpression along axis into subexpressions
      *
@@ -475,11 +560,11 @@ namespace xt
      * @param axis axis along which to split the expression
      */
     template <class E>
-    auto split(E& e, std::size_t n, std::size_t axis = 0)
+    inline auto split(E& e, std::size_t n, std::size_t axis)
     {
         if (axis >= e.dimension())
         {
-            throw std::runtime_error("Split along axis > dimension.");
+            XTENSOR_THROW(std::runtime_error, "Split along axis > dimension.");
         }
 
         std::size_t ax_sz = e.shape()[axis];
@@ -489,7 +574,7 @@ namespace xt
 
         if (rest)
         {
-            throw std::runtime_error("Split does not result in equal division.");
+            XTENSOR_THROW(std::runtime_error, "Split does not result in equal division.");
         }
 
         std::vector<decltype(strided_view(e, sv))> result;
@@ -501,6 +586,38 @@ namespace xt
         return result;
     }
 
+    /**
+     * @brief Split an xexpression into subexpressions horizontally (column-wise)
+     *
+     * This method is equivalent to ``split(e, n, 1)``. 
+     *
+     * @param e input xexpression
+     * @param n number of elements to return
+     */
+    template <class E>
+    inline auto hsplit(E& e, std::size_t n)
+    {
+        return split(e, n, std::size_t(1));
+    }
+
+    /**
+     * @brief Split an xexpression into subexpressions vertically (row-wise)
+     *
+     * This method is equivalent to ``split(e, n, 0)``.
+     *
+     * @param e input xexpression
+     * @param n number of elements to return
+     */
+    template <class E>
+    inline auto vsplit(E& e, std::size_t n)
+    {
+        return split(e, n, std::size_t(0));
+    }
+
+    /***********************
+     * flip implementation *
+     ***********************/
+
     /**
      * @brief Reverse the order of elements in an xexpression along the given axis.
      * Note: A NumPy/Matlab style `flipud(arr)` is equivalent to `xt::flip(arr, 0)`,
@@ -521,7 +638,7 @@ namespace xt
         std::copy(e.shape().cbegin(), e.shape().cend(), shape.begin());
 
         get_strides_t<shape_type> strides;
-        decltype(auto) old_strides = detail::get_strides(e);
+        decltype(auto) old_strides = detail::get_strides<XTENSOR_DEFAULT_LAYOUT>(e);
         resize_container(strides, old_strides.size());
         std::copy(old_strides.cbegin(), old_strides.cend(), strides.begin());
 
@@ -531,6 +648,10 @@ namespace xt
         return strided_view(std::forward<E>(e), std::move(shape), std::move(strides), offset);
     }
 
+    /************************
+     * rot90 implementation *
+     ************************/
+
     template <std::ptrdiff_t N>
     struct rot90_impl;
 
@@ -596,14 +717,14 @@ namespace xt
      *
      * @return returns a view with the result of the rotation
      */
-    template <std::ptrdiff_t N = 1, class E>
-    inline auto rot90(E&& e, const std::array<std::ptrdiff_t, 2>& axes = {0, 1})
+    template <std::ptrdiff_t N, class E>
+    inline auto rot90(E&& e, const std::array<std::ptrdiff_t, 2>& axes)
     {
-        auto ndim = std::ptrdiff_t(e.shape().size());
+        auto ndim = static_cast<std::ptrdiff_t>(e.shape().size());
 
         if (axes[0] == axes[1] || std::abs(axes[0] - axes[1]) == ndim)
         {
-            throw std::runtime_error("Axes must be different");
+            XTENSOR_THROW(std::runtime_error, "Axes must be different");
         }
 
         auto norm_axes = forward_normalize<std::array<std::size_t, 2>>(e, axes);
@@ -611,6 +732,203 @@ namespace xt
 
         return rot90_impl<n>()(std::forward<E>(e), norm_axes);
     }
+
+    /***********************
+     * roll implementation *
+     ***********************/
+
+    /**
+     * @brief Roll an expression.
+     * The expression is flatten before shifting, after which the original
+     * shape is restore. Elements that roll beyond the last position are
+     * re-introduced at the first. This function does not change the input
+     * expression.
+     *
+     * @param e the input xexpression
+     * @param shift the number of places by which elements are shifted
+     *
+     * @return a roll of the input expression
+     */
+    template<class E>
+    inline auto roll(E&& e, std::ptrdiff_t shift)
+    {
+        auto cpy = empty_like(e);
+        auto flat_size = std::accumulate(cpy.shape().begin(), cpy.shape().end(), 1L, std::multiplies<std::size_t>());
+        while(shift < 0)
+        {
+            shift += flat_size;
+        }
+
+        shift %= flat_size;
+        std::copy(e.begin(), e.end() - shift,
+                  std::copy(e.end() - shift, e.end(), cpy.begin()));
+
+        return cpy;
+    }
+
+    namespace detail
+    {
+        /**
+         * Algorithm adapted from pythran/pythonic/numpy/roll.hpp
+         */
+
+        template < class To, class From, class S>
+        To roll(To to, From from, std::ptrdiff_t shift, std::size_t axis, S const& shape, std::size_t M)
+        {
+            std::ptrdiff_t dim = std::ptrdiff_t(shape[M]);
+            std::ptrdiff_t offset = std::accumulate(shape.begin() + M + 1, shape.end(), std::ptrdiff_t(1), std::multiplies<std::ptrdiff_t>());
+            if(shape.size() == M + 1)
+            {
+                if (axis == M)
+                {
+                    const auto split = from + (dim - shift) * offset;
+                    for(auto iter = split, end = from + dim * offset; iter != end; iter += offset, ++to)
+                    {
+                        *to = *iter;
+                    }
+                    for(auto iter = from, end = split; iter != end; iter += offset, ++to)
+                    {
+                        *to = *iter;
+                    }
+                }
+                else
+                {
+                    for(auto iter = from, end = from + dim * offset; iter != end; iter += offset, ++to)
+                    {
+                        *to = *iter;
+                    }
+                }
+            }
+            else
+            {
+                if (axis == M)
+                {
+                    const auto split = from + (dim - shift) * offset;
+                    for(auto iter = split, end = from + dim * offset; iter != end; iter += offset)
+                    {
+                        to = roll(to, iter, shift, axis, shape, M + 1);
+                    }
+                    for(auto iter = from, end = split; iter != end; iter += offset)
+                    {
+                        to = roll(to, iter, shift, axis, shape, M + 1);
+                    }
+                }
+                else
+                {
+                    for (auto iter = from, end = from + dim * offset; iter != end; iter += offset)
+                    {
+                        to = roll(to, iter, shift, axis, shape, M + 1);
+                    }
+                }
+            }
+            return to;
+        }
+    }
+
+    /**
+     * @brief Roll an expression along a given axis.
+     * Elements that roll beyond the last position are re-introduced at the first.
+     * This function does not change the input expression.
+     *
+     * @param e the input xexpression
+     * @param shift the number of places by which elements are shifted
+     * @param axis the axis along which elements are shifted.
+     *
+     * @return a roll of the input expression
+     */
+    template<class E>
+    inline auto roll(E&& e, std::ptrdiff_t shift, std::ptrdiff_t axis)
+    {
+        auto cpy = empty_like(e);
+        auto const& shape = cpy.shape();
+        std::size_t saxis = static_cast<std::size_t>(axis);
+        if(axis < 0)
+        {
+            axis += std::ptrdiff_t(cpy.dimension());
+        }
+
+        if(saxis >= cpy.dimension() || axis < 0)
+        {
+            XTENSOR_THROW(std::runtime_error, "axis is no within shape dimension.");
+        }
+
+        const auto axis_dim = static_cast<std::ptrdiff_t>(shape[saxis]);
+        while(shift < 0)
+        {
+            shift += axis_dim;
+        }
+
+        detail::roll(cpy.begin(), e.begin(), shift, saxis, shape, 0);
+        return cpy;
+    }
+
+    /****************************
+     * repeat implementation    *
+     ****************************/
+    namespace detail
+    {
+        template<class E, class R>
+        inline auto make_xrepeat(E&& e, R&& r, typename std::decay_t<E>::size_type axis)
+        {
+            const auto casted_axis = static_cast<typename std::decay_t<E>::size_type>(axis);
+            if (r.size() != e.shape(casted_axis))
+            {
+                XTENSOR_THROW(std::invalid_argument, "repeats must have the same size as the specified axis");
+            }
+            return xrepeat<const_xclosure_t<E>, R>(std::forward<E>(e), std::forward<R>(r), axis);
+        }
+    }
+
+    /**
+     * @brief Repeats elements of an expression along a given axis.
+     *
+     * @param e the input xexpression
+     * @param repeats The number of repetition of each elements. \ref repeats is broadcasted to
+     * fit the shape of the given \ref axis.
+     * @param axis the axis along which to repeat the value
+     *
+     * @return an expression which as the same shape as \ref e, except along the given \ref axis
+     */
+    template <class E>
+    inline auto repeat(E&& e, std::size_t repeats, std::size_t axis)
+    {
+        const auto casted_axis = static_cast<typename std::decay_t<E>::size_type>(axis);
+        std::vector<std::size_t> broadcasted_repeats(e.shape(casted_axis));
+        std::fill(broadcasted_repeats.begin(), broadcasted_repeats.end(), repeats);
+        return repeat(std::forward<E>(e), std::move(broadcasted_repeats), axis);
+    }
+
+    /**
+     * @brief Repeats elements of an expression along a given axis.
+     *
+     * @param e the input xexpression
+     * @param repeats The number of repetition of each elements. The size of \ref repeats 
+     * must match the shape of the given \ref axis.
+     * @param axis the axis along which to repeat the value
+     *
+     * @return an expression which as the same shape as \ref e, except along the given \ref axis
+     */
+    template <class E>
+    inline auto repeat(E&& e, const std::vector<std::size_t>& repeats, std::size_t axis)
+    {
+        return detail::make_xrepeat(std::forward<E>(e), repeats, axis);
+    }
+
+    /**
+     * @brief Repeats elements of an expression along a given axis.
+     *
+     * @param e the input xexpression
+     * @param repeats The number of repetition of each elements. The size of \ref repeats 
+     * must match the shape of the given \ref axis.
+     * @param axis the axis along which to repeat the value
+     *
+     * @return an expression which as the same shape as \ref e, except along the given \ref axis
+     */
+    template <class E>
+    inline auto repeat(E&& e, std::vector<std::size_t>&& repeats, std::size_t axis)
+    {
+        return detail::make_xrepeat(std::forward<E>(e), std::move(repeats), axis);
+    }
 }
 
 #endif
